# -*- coding: utf-8 -*-
#
#       httpd.py
#
#       Copyright 2013 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import os
import module
import templater

"""
Class httpd handling httpd configuration manipulation
@author: Adam Schubert
@param config: Configuration array
@param handler: Function handling Work statuses
"""


class httpd(module.module):
    """
    Returns virtual hosts path
    @param name: Name of virtual host
    """

    def getVhostPath(self, name):
        name = name.lower()
        return os.path.join(self.config['sites_available'], name)

    """
    Returns enabled virtual hosts path
    @param name: Name of virtual host
    """

    def getEnabledVhostPath(self, name):
        name = name.lower()
        return os.path.join(self.config['sites_enabled'], name)

    """
    Checks if virtual host exists
    @param name: Name of virtual host
    """

    def vhostExists(self, name):
        name = name.lower()
        return os.path.isfile(self.getVhostPath(name))

    """
    Checks if virtual host is enabled
    @param name: Name of virtual host
    """

    def isVhostEnabled(self, name):
        name = name.lower()
        return os.path.isfile(self.getEnabledVhostPath(name))

    """
    Adds virtual hosts
    @param name: Name of virtual host
    """

    def addVhost(self, baseDomain, name, nameAlias, path):
        name = name.lower()
        baseDomain = baseDomain.lower()
        nameAlias = nameAlias.lower()
        if self.vhostExists(name):
            raise Exception('Domain {} already exists!'.format(name))

        config = templater.build('httpd.addVhost',
                                 {'domain': baseDomain, 'ip': '*', 'path': path, 'name': name, 'nameAlias': nameAlias})
        try:
            f = open(self.getVhostPath(name), "w")
            try:
                f.write(config)
            finally:
                f.close()
        except IOError:
            raise Exception('Failed to write {} virtual host configuration file!'.format(name))

    """
    Removes virtual host
    @param name: Name of virtual host
    """

    def removeVhost(self, name):
        name = name.lower()
        if self.isVhostEnabled(name):
            self.disableVhost(name)

        if self.vhostExists(name):
            os.remove(self.getVhostPath(name))

    """
    Disables virtual host
    @param name: Name of virtual host
    """

    def disableVhost(self, name):
        name = name.lower()
        if not self.isVhostEnabled(name):
            raise Exception('Virtual host {} is not enabled!'.format(name))
        os.remove(self.getEnabledVhostPath(name))

    """
    Enables virtual host
    @param name: Name of virtual host
    """

    def enableVhost(self, name):
        name = name.lower()
        if not self.vhostExists(name):
            raise Exception('Virtual host can\'t be enabled, configuration for virtual host {} not found!'.format(name))
        if not self.isVhostEnabled(name):
            raise Exception('Virtual host {} is already enabled!'.format(name))

        os.symlink(self.getVhostPath(name), self.getEnabledVhostPath(name))

    def enableVhostSsl(self, name):
        name = name.lower()
        if not self.vhostExists(name):
            raise Exception('SSL can be enabled only on existing host!')

    def disableVhostSsl(self, name):
        name = name.lower()
        if not self.vhostExists(name):
            raise Exception('SSL can be disabled only on existing host!')
