# -*- coding: utf-8 -*-
#
#       mysqld.py
#
#       Copyright 2013 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import module
import MySQLdb

"""
Class module handling mysqld configuration manipulation
@author: Adam Schubert
@param config: Configuration array
@param handler: Function handling Work statuses
"""


class mysqld(module.module):
    connection = None

    def __init__(self, config, handler):
        module.module.__init__(self, config, handler)
        try:
            self.connection = MySQLdb.connect(self.config['host'], self.config['username'], self.config['password'])
        except Exception:
            raise Exception('Failed to connect to %s' % (self.config['host']))

    """
    Adds new database
    @param name: name of new database
    @param collate: collate of new database
    """

    def addDatabase(self, name, collate='utf8_general_ci'):
        cursor = self.connection.cursor()
        try:
            command = "CREATE DATABASE `{}` COLLATE '{}';".format(name, collate)
            cursor.execute(command)
            self.connection.commit()
        except Exception as e:
            self.connection.rollback()
            raise e

    """
    Adds new user to database
    @param username: username of new user
    @param password: password for new user
    @param host: host from where user will connects
    """

    def addUser(self, username, password, host='localhost'):
        cursor = self.connection.cursor()
        try:
            command = "CREATE USER '{}'@'{}' IDENTIFIED BY '{}';".format(username, host, password)
            cursor.execute(command)
            self.connection.commit()
        except Exception as e:
            self.connection.rollback()
            raise e
        self.flush()

    """
    Grants rights of user on database
    @param username: username of user to grant rights
    @param table: on what table grant this rights
    @param host: host of user to grant a rights
    """

    def grantRights(self, username, database, host='localhost'):
        rights = ['ALTER', 'CREATE', 'CREATE VIEW', 'DELETE', 'DROP', 'INDEX', 'INSERT', 'REFERENCES', 'SELECT',
                  'SHOW VIEW', 'TRIGGER', 'UPDATE']
        cursor = self.connection.cursor()
        try:
            command = "GRANT {} ON `{}`.* TO '{}'@'{}';".format(', '.join(rights), database, username, host)
            cursor.execute(command)
            self.connection.commit()
        except Exception as e:
            self.connection.rollback()
            raise e
        self.flush()

    """
    Removes user acces from database
    @param username: username of user to remove
    @param host: host of user to remove
    """

    def delUser(self, username, host='localhost'):
        cursor = self.connection.cursor()
        try:
            command = "DROP USER '{}'@'{}';".format(username, host)
            cursor.execute(command)
            self.connection.commit()
        except Exception as e:
            self.connection.rollback()
            raise e
        self.flush()

    """
    Removes database
    @param name: name of database to remove
    """

    def delDatabase(self, name):
        cursor = self.connection.cursor()
        try:
            command = "DROP DATABASE `{}`;".format(name)
            cursor.execute(command)
            self.connection.commit()
        except Exception as e:
            self.connection.rollback()
            raise e
        self.flush()

    def changeUserPassword(self, user, host, password):
        cursor = self.connection.cursor()
        try:
            command = "SET PASSWORD FOR '{}'@'{}' = PASSWORD('{}');".format(user, host, password)
            cursor.execute(command)
            self.connection.commit()
        except Exception as e:
            self.connection.rollback()
            raise e
        self.flush()

    def changeUserName(self, user, host, newUser):
        cursor = self.connection.cursor()
        try:
            command = "RENAME USER '{}'@'{}' TO '{}'@'{}';".format(user, host, newUser, host)
            cursor.execute(command)
            self.connection.commit()
        except Exception as e:
            self.connection.rollback()
            raise e
        self.flush()

    def changeUserServer(self, user, host, server):
        cursor = self.connection.cursor()
        try:
            command = "RENAME USER '{}'@'{}' TO '{}'@'{}';".format(user, host, user, server)
            cursor.execute(command)
            self.connection.commit()
        except Exception as e:
            self.connection.rollback()
            raise e
        self.flush()

    def changeDatabaseName(self, name, newName):
        cursor = self.connection.cursor()
        try:
            # fetch info we need
            command = "SELECT `DEFAULT_COLLATION_NAME` FROM `information_schema`.`SCHEMATA` WHERE `SCHEMA_NAME` = '{}';".format(
                name)
            cursor.execute(command)
            collation = cursor.fetchone()[0]

            command = "SELECT TABLE_NAME FROM `information_schema`.`tables` WHERE `TABLE_SCHEMA` = '{}'".format(name)
            cursor.execute(command)
            tables = cursor.fetchall()

            commands = []
            commands.append("CREATE DATABASE `{}` COLLATE '{}';".format(newName, collation))
            for table in tables:
                commands.append("RENAME TABLE `{}`.`{}` TO `{}`.`{}`".format(name, table[0], newName, table[0]))

            commands.append("UPDATE `mysql`.`db` SET `Db` = '{}' WHERE Db='{}'".format(newName, name));

            commands.append("DROP DATABASE `{}`;".format(name))

            for command in commands:
                cursor.execute(command)
            self.connection.commit()
        except Exception as e:
            self.connection.rollback()
            raise e
        self.flush()

    """
    Flushes priviliges
    """

    def flush(self):
        cursor = self.connection.cursor()
        try:
            command = "FLUSH PRIVILEGES;"
            cursor.execute(command)
            self.connection.commit()
        except Exception as e:
            self.connection.rollback()
            raise e
