# -*- coding: utf-8 -*-
#
#       passwd.py
#
#       Copyright 2013 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import os
import collections
import backuper
import shutil
import datetime

"""
Class passwd handling passwd config file manipulation
@author: Adam Schubert
"""


class passwd:
    passwdpath = '/etc/passwd'  # This is UNIX standard, no need to put it in config

    openedFile = {}
    oldFile = ""

    def __init__(self):
        try:
            with open(self.passwdpath, 'r') as f:
                for line in f:
                    self.oldFile += line
                    # adam:x:1000:100:Adam Schubert,,,:/home/adam:/bin/bash
                    splited = line.split(':')
                    if len(splited) == 7:
                        username, x, u_id, g_id, description, home, bash = splited
                        self.openedFile.update({username: {'x': x, 'uId': int(u_id), 'gId': int(g_id),
                                                           'description': description, 'home': home, 'bash': bash}})
                    else:
                        print('Malformed ' + self.passwdpath + ' detected:' + line)
                self.sortDataStorage()
        except IOError:
            raise Exception('Passwd file {} not found or its not allowed for reading!'.format(self.passwdpath))

    """
    Internal function for data sorting
    """

    def sortDataStorage(self):
        self.openedFile = collections.OrderedDict(sorted(self.openedFile.items()))

    """
    Return new user ID for newly created user
    """

    def getNewUserId(self):
        biggiest = 0
        for row in self.openedFile:
            if self.openedFile[row]['uId'] > biggiest:
                biggiest = self.openedFile[row]['uId']

        return biggiest + 1

    """
    Adds user to passw file
    @param username: username of new user
    @param home: home dir of new user
    @param bash: bash of new user
    @param gId: group ID of new user
    """

    def addUser(self, username, gId, home=None, bash='/bin/bash'):
        # no home specified, lets use defaults + username
        if not home:
            home = '/home/' + username

        # check username
        if username in self.openedFile:
            raise Exception('Username {} already exists!'.format(username))

        u_id = self.getNewUserId()
        # check home
        home_exists = os.path.isdir(home)
        for row in self.openedFile:
            if self.openedFile[row]['home'] == home:
                home_exists = True

        if not home_exists:
            self.addHomeDir(home, u_id, gId)

        # Add new user
        self.openedFile.update({username: {'x': 'x', 'uId': u_id, 'gId': gId,
                                           'description': datetime.date.today().strftime('%Y-%m-%d %H-%M-%S'),
                                           'home': home, 'bash': bash + "\n"}})
        self.sortDataStorage()

    """
    Removes user from passw file
    @param username: username of removed user
    """

    def delUser(self, username):
        if username in self.openedFile:
            del self.openedFile[username]
        else:
            raise Exception('Username {} not found!'.format(username))

    """
    Changes user home Path
    @param username: usernam of user to change a path
    @param home: new home path to set
    """

    def changeHome(self, username, home):

        if not self.user_exists(username):
            raise Exception('User {} not found!'.format(username))

        user = self.openedFile[username]

        home_exists = os.path.isdir(home)
        for row in self.openedFile:
            if self.openedFile[row]['home'] == home:
                home_exists = True

        if not home_exists:
            self.addHomeDir(home, user['uId'], user['gId'])

        self.openedFile[username]['home'] = home

    """
    Changes users name
    @param username: old username to change
    @param newUsername: new username to change to
    """

    def changeUsername(self, username, newUsername):
        try:
            self.openedFile[newUsername] = self.openedFile.pop(username)
        except KeyError:
            raise Exception('User {}, not found ?'.format(username))

    """
    Saves passwd file after modifications are made
    """

    def save(self):
        file_content = ""
        for row in self.openedFile:
            data = self.openedFile[row]
            file_content += ':'.join(
                [row, data['x'], str(data['uId']), str(data['gId']), data['description'], data['home'], data['bash']])

        # create backup before we try to owerwrite it
        backup = backuper.Backuper(self.passwdpath)
        backup.add_backup(self.passwdpath)

        try:
            with open(self.passwdpath, 'w') as passwdFile:
                passwdFile.write(file_content)
        except IOError:
            raise Exception('Cannot write to passwd file {}, is writtable ?'.format(self.passwdpath))

    """
    Adds home dir for user
    @param path: path of newly created homedir
    @param uId: user ID of newly created homedir
    @param gId: group ID of newly created homedir
    """

    def addHomeDir(self, path, uId, gId):
        if os.path.isdir(path):
            raise Exception('Home dir {} already exists !'.format(path))
        else:
            os.mkdir(path, 0o770)
            os.chown(path, uId, gId)

    """
    Removes homedir, use with caution
    @param path: home dir path to delete
    """

    def delHomeDir(self, path):
        if os.path.isdir(path):
            shutil.rmtree(path)
        else:
            raise Exception('Home dir {} not found !'.format(path))

    """
    Checks if user exists in passwd file
    @param username: Username to check
    """

    def user_exists(self, username):
        if username in self.openedFile:
            return True
        else:
            return False
