# -*- coding: utf-8 -*-
#
#       proftpd.py
#
#       Copyright 2013 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import re
import collections
import backuper

"""
Class proftpd handling proftp config parsing
@author: Adam Schubert
@param ftpHome: default root where ftp operates
@param configFile: absolute path to proftp config file
"""


class proftpd:
    config_users = ""
    config = ""
    configFile = '/etc/proftpd/proftpd.conf'
    parsed_config_users = {}

    def __init__(self, config_file=None):

        if config_file:
            self.configFile = config_file

        self.processConf()
        self.parseConfUsers()

    """
    Parses config two global values, config header woth global config, and user config section
    """

    def processConf(self):
        try:
            with open(self.configFile, 'r') as f:
                found_confi_start = False
                for line in f:
                    # add only section after first <Directory
                    if line.startswith("<Directory"):
                        found_confi_start = True
                    if found_confi_start:
                        # ignore comments
                        if not line.startswith("#"):
                            self.config_users += line
                    else:
                        self.config += line
        except IOError:
            raise Exception('Config file {} not found or its not allowed for reading!'.format(self.configFile))

    """
    Parses user config section into array
    """

    def parseConfUsers(self):
        regex_dir = re.compile("<Directory (.*?)>(.*?)</Directory>", re.IGNORECASE | re.MULTILINE | re.DOTALL)
        regex_limit = re.compile("<Limit (.*?)>.*?(.*?)</Limit>", re.IGNORECASE | re.MULTILINE | re.DOTALL)
        regex_a = re.compile("</Limit>(.*)", re.IGNORECASE | re.MULTILINE | re.DOTALL)

        found_dirs = regex_dir.findall(self.config_users)
        for found_dir in found_dirs:
            found_limits = regex_limit.findall(found_dir[1])
            founda = regex_a.findall(found_dir[1])

            path = found_dir[0].strip()

            users = {}
            for user in found_limits:
                parsed = user[1].strip().split(' ')

                if len(parsed) == 2:
                    action, username = parsed
                    users.update({username: {'rights': user[0], 'action': action}})
                else:
                    users.update({'': {'rights': user[0], 'action': parsed[0]}})

            parsed = founda[0].strip().split(' ')
            if parsed:
                if len(parsed) == 2:
                    allow, state = parsed
                else:
                    allow = parsed[0]
                    state = ''
                self.parsed_config_users.update({path: {'users': users, 'scrap': {'allow': allow, 'state': state}}})
        # sort Datastorage
        self.sortDataStorage()

    """
    Saves parsed file with changes
    """

    def save(self):
        config_file = self.config
        config_file += """
#################################################################
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
#! Section bellow is parsed by script, do not edit it manualy! !#
#! Sekce níže je pársována scriptem, neprovádějte úpravy ručně!!#
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
#                                                               #
# If you need unsupported configuration by parsing script,      #
# please contact script administrator                           #
#################################################################
    """
        for path in self.parsed_config_users:
            print(self.parsed_config_users[path])
            limits = ""
            for user in self.parsed_config_users[path]['users']:
                limits += """
  <Limit {0}>
    {1} {2}
  </Limit>
  """.format(self.parsed_config_users[path]['users'][user]['rights'],
             self.parsed_config_users[path]['users'][user]['action'], user)

            config_file += """
<Directory {0}>
  {1}
  {2} {3}
</Directory>
""".format(path, limits, self.parsed_config_users[path]['scrap']['allow'], self.parsed_config_users[path]['scrap']['state'])

        # create backup before overwrite
        backup = backuper.Backuper(self.configFile)
        backup.add_backup(self.configFile)

        try:
            with open(self.configFile, 'w') as new_file:
                new_file.write(config_file)
        except IOError:
            raise Exception('Cannot write to config file {}, is writtable ?'.format(self.configFile))

    """
    Internal sorter to sort data by path
    """

    def sortDataStorage(self):
        self.parsed_config_users = collections.OrderedDict(sorted(self.parsed_config_users.items()))

    """
    Deletes user from config
    @param username: username to delete from confi
    """

    def delUser(self, username):
        # unicode convert
        username = username.encode('utf-8')
        # first we do a chek, if there is such a user and if is unique
        found = 0
        for path in self.parsed_config_users:
            if username in self.parsed_config_users[path]['users']:
                found += 1

        if found == 0:
            raise Exception('Ftp user {} not found'.format(username))
        elif found > 1:
            raise Exception('Found more then one users witch name {}'.format(username))
        else:
            found_path = ''
            for path in self.parsed_config_users:
                # delete user and get his path to check if delete whole Dir
                if username in self.parsed_config_users[path]['users']:
                    found_path = path
                    del self.parsed_config_users[path]['users'][username]

            if found_path in self.parsed_config_users:
                # there is no eny other user witch can use this dir, delte whole dir
                if len(self.parsed_config_users[found_path]['users']) == 0:
                    del self.parsed_config_users[found_path]
        # sort Datastorage
        self.sortDataStorage()

    """
    Adds user into config
    @param path: path to target a FTP
    @param username: username of new FTP accces
    @param rights: defines user rights
    @param action: action
    """

    def addUser(self, username, path, rights=u'ALL', action=u'AllowUser'):
        # convert to strings
        path = path.encode('utf8')
        username = username.encode('utf8')
        rights = rights.encode('utf8')
        action = action.encode('utf8')
        # path exists, we are adding...
        if path in self.parsed_config_users:
            if username in self.parsed_config_users[path]['users']:
                raise Exception('User {} exists for path {}'.format(username, path))
            else:
                self.parsed_config_users[path]['users'].update({username: {'rights': rights, 'action': action}})
        # path not found, creating new record
        else:
            self.parsed_config_users.update({path: {'users': {username: {'rights': rights, 'action': action}},
                                                  'scrap': {'allow': 'AllowOverwrite', 'state': 'on'}}})
        # sort Datastorage
        self.sortDataStorage()

    """
    Checks if user exists in config
    @param username: name of user to check
    """

    def user_exists(self, username):
        username = username.encode('utf8')
        found = 0
        for path in self.parsed_config_users:
            if username in self.parsed_config_users[path]['users']:
                found += 1
        if found == 0:
            return False
        else:
            return True
