# -*- coding: utf-8 -*-
#
#       shadowd.py
#
#       Copyright 2013 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import collections
import backuper
import crypt
import string
import random

"""
Class shadowd handling shadow config parsing
@author: Adam Schubert
"""


class shadowd:
    shadowdpath = '/etc/shadow'  # This is UNIX standard, no need to put it in config

    # password levels
    #  ID  | Method
    # ─────────────────────────────────────────────────────────
    # 1     | MD5
    # 2a    | Blowfish (not in mainline glibc; added in some Linux distributions)
    # 5     | SHA-256 (since glibc 2.7)
    # 6     | SHA-512 (since glibc 2.7)
    password_level = 6

    opened_file = {}
    old_file = ""

    def __init__(self):
        try:
            with open(self.shadowdpath, 'r') as f:
                for line in f:
                    self.old_file += line
                    # adam:superpassword:0:0:99999:7:::
                    splited = line.split(':')
                    if len(splited) == 9:
                        username, password, password_last_change, password_may_be_changed, password_must_be_changed, password_expire_warning, till_expire, been_disabled, reserved = splited
                        self.opened_file.update({username: {'password': password,
                                                           'passwordLastChange': password_last_change,
                                                           'passwordMayBeChanged': password_may_be_changed,
                                                           'passwordMustBeChanged': password_must_be_changed,
                                                           'passwordExpireWarning': password_expire_warning,
                                                            'tillExpire': till_expire, 'beenDisabled': been_disabled,
                                                           'reserved': reserved}})
                    else:
                        print('Malformed ' + self.shadowdpath + ' detected:' + line)
                self.sortDataStorage()
        except IOError:
            raise Exception('Shadow file {} not found or its not allowed for reading!'.format(self.shadowdpath))

    """
    Internal sorter sorts config file
    """

    def sortDataStorage(self):
        self.opened_file = collections.OrderedDict(sorted(self.opened_file.items()))

    """
    Generates new salt for new password
    """

    def getSalt(self):
        chars = string.ascii_uppercase + string.ascii_lowercase + string.digits
        size = random.randint(8, 8)
        return ''.join(random.choice(chars) for x in range(size))

    """
    Adds new user to shadow file
    @param username: username to create
    @param password: password to set
    """

    def addUser(self, username, password):

        # check username
        if username in self.opened_file:
            raise Exception('Username {} already exists!'.format(username))

        # Add new user
        self.opened_file.update({username: {
            'password': crypt.crypt(password, '$' + str(self.password_level) + '$' + self.getSalt() + '$'),
            'passwordLastChange': 0, 'passwordMayBeChanged': 0, 'passwordMustBeChanged': 99999,
            'passwordExpireWarning': 7, 'tillExpire': '', 'beenDisabled': '', 'reserved': '' + "\n"}})
        self.sortDataStorage()

    """
    Removes user from shadow file
    @param username: username to delte from shadow file
    """

    def delUser(self, username):
        if username in self.opened_file:
            del self.opened_file[username]
        else:
            raise Exception('Username {} not found!'.format(username))

    """
    Changes password to user
    @param username: username to change password
    @param password: new password to set
    """

    def changePassword(self, username, password):
        if username in self.opened_file:
            self.opened_file[username]['password'] = crypt.crypt(password, '$' + str(
                self.password_level) + '$' + self.getSalt() + '$')
        else:
            raise Exception('Username {} not found!'.format(username))

    """
    Changes users name
    @param username: old username to change
    @param newUsername: new username to change to
    """

    def changeUsername(self, username, newUsername):
        try:
            self.opened_file[newUsername] = self.opened_file.pop(username)
        except KeyError:
            raise Exception('User {}, not found ?'.format(self.shadowdpath))

    """
    Saves modified config
    """

    def save(self):
        file_content = ""
        for row in self.opened_file:
            data = self.opened_file[row]
            file_content += ':'.join(
                [row, data['password'], str(data['passwordLastChange']), str(data['passwordMayBeChanged']),
                 str(data['passwordMustBeChanged']), str(data['passwordExpireWarning']), data['tillExpire'],
                 data['beenDisabled'], data['reserved']])

        # Create backup before overwrite
        backup = backuper.Backuper(self.shadowdpath)
        backup.add_backup(self.shadowdpath)

        try:
            with open(self.shadowdpath, 'w') as shadowFile:
                shadowFile.write(file_content)
        except IOError:
            raise Exception('Cannot write to shadow file {}, is writtable ?'.format(self.shadowdpath))

    """
    Checks if user exists in confi
    @param username: username to check
    """

    def user_exists(self, username):
        if username in self.opened_file:
            return True
        else:
            return False
