#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#       main.py
#
#       Copyright 2013 Adam Schubert <adam.schubert@sg1-game.net>
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

# core imports
import os
import ConfigParser
import json
import sys
import base64
import urllib
import modules
import OpenSSL.crypto as ocrypto
from twisted.internet import reactor, protocol
from twisted.protocols import basic
from twisted.python import log

"""
Class SrorProtocol Defines communication protocol
@author: Adam Schubert
"""


class SrorProtocol(basic.LineReceiver):
    def __init__(self, factory):
        self.factory = factory

    """
    When connection is made (onConnect)
    """

    def connectionMade(self):
        print('Connected, waiting for commands!')
        self.factory.clients.add(self)

    """
    When connection is lost (onDisconnect)
    """

    def connectionLost(self, reason):
        print('Client disconnected: {}'.format(reason))
        self.factory.clients.remove(self)

    """
    When line is received
    """

    def lineReceived(self, line):
        if len(line) > 0:
            parsed_command = json.loads(line)
            data = base64.b64decode(parsed_command['data'])
            signature = base64.b64decode(parsed_command['signature'])
        else:
            raise Exception('No data received!')

        # Special commands mainly used for testing via telnet
        if data.find('exit') > -1:
            self.transport.loseConnection()

        # Standard communication
        try:
            ocrypto.verify(self.factory.bootstrap.key, signature, data, 'sha1')

            # Continue in work!
            print('{} Signature is ok!'.format(self.transport.getHost()))

            # parse data
            command = json.loads(data)
            if command:
                if command['service'] in self.factory.bootstrap.activeServices:
                    try:
                        data = self.factory.bootstrap.activeServices[command['service']].addWork(command)
                        self.sendLine(
                            json.dumps({'status': True, 'code': 0, 'message': 'Work added into front!', 'data': data}))
                    except Exception as e:
                        self.sendLine(json.dumps({'status': False, 'code': 5, 'message': str(e), 'data': data}))
                else:
                    raise Exception(
                        'Service {} is not implemented or its module is missing '.format(command['service']))
            else:
                raise Exception('Command is malformed!')

        except ocrypto.Error as signatureError:
            print('{} Wrong signature!'.format(self.transport.getHost()))
            self.sendLine(json.dumps({'status': False, 'code': 1, 'message': str(signatureError)}))
        except Exception as e:
            print(e)
            print('{} {}'.format(self.transport.getHost(), str(e)))
            self.sendLine(json.dumps({'status': False, 'code': 4, 'message': str(e)}))


"""
Class ServerRor defines our server
"""


class ServerRor(protocol.Factory):
    def __init__(self, bootstrap):
        self.clients = set()
        self.bootstrap = bootstrap

    """
    Build communication protocol
    """

    def buildProtocol(self, addr):
        return SrorProtocol(self)


"""
Class main, beggining of the end :)
"""


class main():
    config = None
    key = None
    activeServices = {}

    def __init__(self):
        """
        First of all we load main config
        """
        # Load config /etc/serverror.conf
        self.loadConfig()

        if self.config.getboolean('MAIN', 'logging'):
            log.startLogging(open('/var/log/serverror.log', 'w'))
        else:
            log.startLogging(sys.stdout)

        print('Running on pid {}'.format(os.getpid()))
        """
        Then init all needed components and run socket it self
        """
        self.loadCertificate()

        self.startServices()
        self.startSocket()

    """
    Checks and loads module
    @param name: name of module
    """

    def isModuleLoaded(self, name):
        if hasattr(self, name):
            print(getattr(self, name))
            return hasattr(getattr(self, name), name)

    """
    Loads config from main config file
    """

    def loadConfig(self):
        configSearchPaths = ['/etc/serverror/serverror.conf', 'etc/serverror/serverror.conf']
        configName = configSearchPaths[0]
        # determine where config file is located...
        for path in configSearchPaths:
            if os.path.isfile(path):
                configName = path
                break

        if os.path.isfile(configName) == False:
            raise Exception('Failed to open configuration file {} '.format(configName))
        self.config = ConfigParser.ConfigParser()
        self.config.read(configName)
        print('Configuration loaded...')

    """
    Loads cert
    """

    def loadCertificate(self):
        cerPath = self.config.get('MAIN', 'certificate')
        try:
            with open(cerPath, 'r') as f:
                cert = f.read()
            self.key = ocrypto.load_certificate(ocrypto.FILETYPE_PEM, cert)
            print('Certificate loaded...')
        except IOError:
            raise Exception('Failed to open certificate {} '.format(cerPath))

    """
    Inicializes services specified in configuration
    """

    def startServices(self):
        for section in self.config.sections():
            if self.config.getboolean(section, 'enabled') == True:
                if hasattr(modules, section):
                    # Dont pass config object to module, just unpack it and send as array
                    config = {}
                    for item in self.config.items(section):
                        config[item[0]] = item[1]

                    try:
                        self.activeServices[section] = getattr(modules, section)(config, self.workFinished)
                    except Exception as e:
                        raise Exception('Failed to initiate module {} with error: "{}"'.format(section, str(e)))
                else:
                    print('Module {} not found! Please fix your configuration.'.format(section))

    """
    Handler when work is finished...
    @param info: info
    @param status: bool
    """

    def workFinished(self, work, status, message):
        # Call notify
        if 'notification' in work:
            try:
                data = {'message': message, 'status': status}
                print(urllib.urlopen(work['notification'] + '&' + urllib.urlencode({'data': json.dumps(data)})).read(
                    1000))
            except Exception as e:
                print(str(e))
        if status:
            print(work)
        else:
            raise Exception(message)

    """
    Opens socet for our server
    """

    def startSocket(self):
        factory = ServerRor(self)
        reactor.listenTCP(self.config.getint('MAIN', 'port'), factory)
        reactor.run()


if __name__ == "__main__":
    main()
